unit BufferedIO;

interface

const
	// the reading chunk size
	ChunkSize = 2048;
	
	// a line break
	LineBreak = #10;

type
	// one chunk read from the file
	Chunk = array[1..ChunkSize] of Char;
	
	// a text file read in a buffered manner
	BufferedTextFile = Record
		DataFile : File of Chunk; // a data file made of Chunk instances
		ReadBuffer : AnsiString;  // the read buffer
	end;
	
	BufferedTextFilePtr = ^BufferedTextFile;
		
// opens a buffered text file
function OpenBufferedFile(const Path : String) : BufferedTextFilePtr;

// closes a buffered text file and frees all resources allocated for it
procedure CloseBufferedFile(const f : BufferedTextFilePtr);

// reads one line (an ANSI string) from a buffered file
function BufferedReadLine(const f : BufferedTextFilePtr) : AnsiString;

implementation

function OpenBufferedFile(const Path : String) : BufferedTextFilePtr;
var
	NewBufferedFile : BufferedTextFilePtr = nil;
begin
	New(NewBufferedFile);

	// open the file
	Assign(NewBufferedFile^.DataFile, path);
	
	{-I}
	Reset(NewBufferedFile^.DataFile);
	{+I}
	
	// return NULL on I/O errors
	if 0 <> IOResult then
	begin
		Dispose(NewBufferedFile);
		OpenBufferedFile := nil;
		exit;
	end;
	
	OpenBufferedFile := NewBufferedFile;
end;
	
procedure CloseBufferedFile(const f : BufferedTextFilePtr);
begin
	// close the data file
	Close(f^.DataFile);
	
	// free the BufferedTextFile instance
	Dispose(f);
end;
	
function BufferedReadLine(const f : BufferedTextFilePtr) : AnsiString;
var
	Line : AnsiString = '';
	Position : Integer = 0;
	ReadChunk : Chunk = '';
begin
	// check whether the read buffer contains at least one line
	Position := Pos(LineBreak, f^.ReadBuffer);

	// read enough chunks to form at list one line
	if 0 = Position then
	begin
		while (0 = Position) and (not EOF(f^.DataFile)) do
		begin	
			// read a chunk
			Read(f^.DataFile, ReadChunk);
			
			// append the chunk to the reading buffer
			f^.ReadBuffer := f^.ReadBuffer + ReadChunk;
	
			// locate the line break, if there is any
			Position := Pos(LineBreak, f^.ReadBuffer);
		end;
	end;
	
	// filter one line
	Line := Copy(f^.ReadBuffer, 1, Position);
	
	// remove the line from the read buffer
	f^.ReadBuffer := Copy(f^.ReadBuffer, 1 + Position, 
	                      Length(f^.ReadBuffer) - Position - 1);
	                   
	BufferedReadLine := Line;
end;

begin
end.
