unit Parser;
	
interface

uses
	Ppm, BufferedIO, SysUtils;
	
const
	// the separator between a field and its value
	FieldValueSeparator : Char = ':' ;

// returns a token from a delimeted string
function GetStrToken(const Line : AnsiString; 
                     const Delim : Char; 
                     const Index : Integer) : AnsiString;

// returns the field specified in a line
function GetField(const Line : AnsiString) : String;

// returns the value of a field specified in a line
function GetValue(const Line : AnsiString) : AnsiString;

// trims and formats a single dependency
function FormatDependency(const Dependency: String; 
                          const Prefix: Char) : String;

// converts a list of dependencies or conflicts into a PPM one
function FormatDependencies(const Dependencies : AnsiString; 
                            const Prefix : char) : AnsiString;

// parses a package list
function Parse(const Path : String; 
               const Print : Boolean;
               const Distro : String;
               const DistroVersion : String;
               const Repo : String) : PackagePtr;

implementation

function GetStrToken(const Line : AnsiString; 
                     const Delim : Char; 
                     const Index : Integer) : AnsiString;
var
	Position : Integer = 0;
	Counter : Integer = 0;
	SplitLine : AnsiString = '';
begin
	// create a copy of the original string
	SplitLine := Line;

	// strip tokens
	for Counter := 1 to (Index - 1) do
	begin
		// locate the separator
		Position := Pos(Delim, SplitLine);
		
		if 0 = Position then
		begin
			GetStrToken := '';
			exit;
		end;
		
		// strip one token
		SplitLine := Copy(SplitLine, 1 + Position, 
		                  Length(Line) - Position);	
	end;
	
	// locate another delimeter
	Position := Pos(Delim, SplitLine);

	// filter the first token	
	if 0 = Position then
		GetStrToken := SplitLine
	else
		GetStrToken := Copy(SplitLine, 1, Position - 1)
end;

function GetField(const Line : AnsiString) : String;
	var Field : String = '';
begin
	Field := GetStrToken(Line, FieldValueSeparator, 1);

	// filter description lines that don't contain a field
	if ' ' = Field[1] then
		GetField := ''
	else
		GetField := Field
end;

function GetValue(const Line : AnsiString) : AnsiString;
begin
	GetValue := Trim(GetStrToken(Line, FieldValueSeparator, 2));
end;

function FormatDependency(const Dependency: String; const Prefix: Char) : String;
var 
	Position : Integer = 0;
begin
	// locate the first space
	Position := Pos(' ', Dependency);
	
	if 0 = Position then
	begin
		FormatDependency := Prefix + Dependency;
		exit;	
	end;

	FormatDependency := Prefix + Copy(Dependency, 1, Position - 1);
end;

function FormatDependencies(const Dependencies : AnsiString; const Prefix : char) : AnsiString;
var
	Temp : AnsiString = '';
	Dependency : String = '';
	Position : Integer = 0;
	Result : AnsiString = '';
begin
	// filter the first dependency
	Position := Pos(', ', Dependencies);
	
	// if there's just one dependency, process it
	if 0 = Position then
	begin
		FormatDependencies := FormatDependency(Dependencies, Prefix);
		exit;
	end;
	
	// otherwise, process all dependencies
	Temp := Dependencies;
	
	while 0 <> Position do
	begin
		// process one dependency
		Dependency := FormatDependency(Copy(Temp, 1, Position - 1),
		                               Prefix);
		                               
		// append the formatted dependency to the result
		if '' = Result then
			Result := Dependency
		else
			Result := Result + ',' + Dependency;
									
		// strip the dependency
		Delete(Temp, 1, Position + 1);
		
		// repeat!
		Position := Pos(',', Temp);
	end;

	FormatDependencies := Result;
end;

function Parse(const Path : String; 
               const Print : Boolean;
               const Distro : String;
               const DistroVersion : String;
               const Repo : String) : PackagePtr;
var
	PackageList : BufferedTextFilePtr = nil;
	Line : AnsiString = '';
	Packages : PackagePtr = nil;
	CurrentPackage : PackagePtr = nil;
	Field : String = '';
	Value : AnsiString = '';
	Position : integer = 0;
begin
	// open the text file
	PackageList := OpenBufferedFile(Path);
	
	// on failure, return a NULL pointer
	if nil = PackageList then
	begin
		Parse := nil;
		exit;
	end;
		
	// read the package list
	repeat
		// read one line, in a buffered manner
		Line := BufferedReadLine(PackageList);
		
		// filter the field
		Field := GetField(Line);
		
		if 'Package' = Field then
		begin
			// create the new package
			CurrentPackage := CreatePackage(GetValue(Line));
			
			// append the package to the package list
			if nil = Packages then
				Packages := CurrentPackage
			else
				GetLastPackage(Packages)^.Next := CurrentPackage;

			Continue;			
		end;
			
		// parse the category field
		if 'Section' = Field then
		begin	
			CurrentPackage^.Category := GetValue(Line);
			Continue;
		end;		
		
		// parse the size field
		if 'Installed-Size' = Field then
		begin	
			CurrentPackage^.Size := GetValue(Line) + 'K';
			Continue;
		end;		
			
		// parse the version field
		if 'Version' = Field then
		begin
			// locate the revision
			Value := GetValue(Line);
			Position := Pos('-', Value);
			
			CurrentPackage^.Version := Copy(Value, 1, Position - 1);
			CurrentPackage^.Revision := Copy(Value, 1 + Position, 
			                                 Length(Value) - Position);
			Continue;			
		end;
	
		// parse the dependencies field
		if 'Depends' = Field then
		begin
			CurrentPackage^.Dependencies := FormatDependencies(
			                                             GetValue(Line), 
			                                             '+');			
			Continue;			
		end;
		
		// parse the conflicts field
		if 'Conflicts' = Field then
		begin
			CurrentPackage^.Conflicts := FormatDependencies(
			                                             GetValue(Line), 
			                                             '-');
			Continue;			
		end;		
		
		// parse the path and file name fields
		if 'Filename' = Field then
		begin
			// locate the file name
			Value := GetValue(Line);
			Position := Pos('-', Value);
			
			CurrentPackage^.FileName := ExtractFileName(Value);
			CurrentPackage^.Path := Value;
			
			// delete the file name from the path field
			Delete(CurrentPackage^.Path, 
			       Pos(CurrentPackage^.FileName, 
			           CurrentPackage^.Path) - 1, 
			       Length(CurrentPackage^.Path));
			
			Continue;			
		end;

		// parse the description field
		if 'Description' = Field then
		begin
			CurrentPackage^.Description := GetValue(Line);
			
			// if requested, print the package entry
			if Print then
				WriteLn(PackageToEntry(CurrentPackage,
				                       Distro,
				                       DistroVersion,
				                       Repo));
			
			Continue;		
		end;	
		
	until '' = Line;
	
	Parse := Packages;
end;

begin
end.
