unit Ppm;

interface

uses
	SysUtils;
	
type
	// an object which represents a package
	PackagePtr = ^Package;
	Package = Record
		Name : String;
		Version : String;
		Revision : String;
		Category : String;
		Size : String;
		Path : String;
		FileName : String;
		Dependencies : AnsiString;
		Conflicts : AnsiString;
		Description : AnsiString;
		Next : PackagePtr;
	end;

// creates a new Package instance
function CreatePackage(const Name : String) : PackagePtr;

// frees a Package instance
procedure DestroyPackage(PackagePointer : PackagePtr);

// frees an entire linked list of Package instances
procedure DestroyPackageList(PackageList : PackagePtr);

{ returns a pointer to the last package in a linked list of Package 
  instances }
function GetLastPackage(const PackageList : PackagePtr) : PackagePtr;

{ creates a PPM package entry from a Package instance, see
  http://www.bkhome.org/woof/pkg-db-format.htm }
function PackageToEntry(const PackagePointer : PackagePtr;
                        const Distro : String;
                        const DistroVersion : String;
                        const Repo : String) : AnsiString;
implementation

function CreatePackage(const Name : String) : PackagePtr;
var
	NewPackage : PackagePtr = nil;
begin
	// create the new package
	New(NewPackage);

	// initialize all fields
	NewPackage^.Name := Name;
	NewPackage^.Version := '';
	NewPackage^.Category := '';
	NewPackage^.Size := '';
	NewPackage^.Path := '';
	NewPackage^.FileName := '';
	NewPackage^.Dependencies := '';
	NewPackage^.Conflicts := '';
	NewPackage^.Description := '';
	NewPackage^.Next := nil;
	
	CreatePackage := NewPackage;
end;

procedure DestroyPackage(PackagePointer : PackagePtr);
begin
	Dispose(PackagePointer);
end;

procedure DestroyPackageList(PackageList : PackagePtr);
var
	CurrentPackage : PackagePtr = nil;
	NextPackage : PackagePtr = nil;
begin
	CurrentPackage := PackageList;
	
	// free each node in the linked list
	while nil <> CurrentPackage do
	begin
		NextPackage := CurrentPackage^.Next;
		DestroyPackage(CurrentPackage);	
		CurrentPackage := NextPackage;
	end;
end;

function GetLastPackage(const PackageList : PackagePtr) : PackagePtr;
var
	CurrentPackage : PackagePtr = nil;
begin
	// when a NULL pointer is received, return NULL
	if nil = PackageList then
	begin
		GetLastPackage := nil;
		exit;
	end;
	
	CurrentPackage := PackageList;
	
	// locate the last package
	while nil <> CurrentPackage^.Next do
		CurrentPackage := CurrentPackage^.Next;
		
	GetLastPackage := CurrentPackage;
end;

function PackageToEntry(const PackagePointer : PackagePtr;
                        const Distro : String;
                        const DistroVersion : String;
                        const Repo : String) : AnsiString;
var
	EntryFormat : String = '%s-%s-%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s';
	Dependencies : AnsiString = '';
begin
	// merge the dependencies and the conflicts into one list
	if '' = PackagePointer^.Dependencies then
		Dependencies := PackagePointer^.Conflicts
	else
	begin
		if '' = PackagePointer^.Conflicts then
			Dependencies := PackagePointer^.Dependencies
		else
			Dependencies := PackagePointer^.Dependencies + ',' + 
			                PackagePointer^.Conflicts
	end;

	PackageToEntry := Format(EntryFormat,
	                         [ PackagePointer^.Name,
	                           PackagePointer^.Version,
	                           PackagePointer^.Revision,
	                           PackagePointer^.Name,
	                           PackagePointer^.Version,
	                           PackagePointer^.Revision,
	                           PackagePointer^.Category,
	                           PackagePointer^.Size,
	                           PackagePointer^.Path,
	                           PackagePointer^.FileName,
	                           Dependencies,
	                           PackagePointer^.Description,
	                           Distro,
	                           DistroVersion,
	                           Repo ]);                           
end;

begin
end.
